variable "external_dns_k8s_namespace" {
  type = string
}

variable "external_dns_k8s_service_account" {
  type = string
}

variable "external_dns_gcp_service_account_email" {
  type = string
}

locals {
  external_dns_namespace     = length(kubernetes_namespace.external_dns) == 0 ? var.external_dns_k8s_namespace : kubernetes_namespace.external_dns[0].metadata[0].name
  external_dns_domain_filter = trimsuffix("${var.cluster.name}.${var.dns_managed_zone.dns_name}", ".")
}

resource "kubernetes_namespace" "external_dns" {
  count = var.external_dns_k8s_namespace != "default" ? 1 : 0
  metadata {
    name = var.external_dns_k8s_namespace
  }

  timeouts {
    delete = "60m"
  }

  depends_on = [var.node_pool]
}


resource "kubernetes_service_account" "external_dns" {
  metadata {
    name      = var.external_dns_k8s_service_account
    namespace = local.external_dns_namespace
    annotations = {
      "iam.gke.io/gcp-service-account" = var.external_dns_gcp_service_account_email
    }
  }

  depends_on = [var.node_pool]
}

resource "kubernetes_cluster_role" "external_dns" {
  metadata {
    name = "external-dns"

    labels = {
      "app.kubernetes.io/name" = "external-dns"
    }
  }

  rule {
    verbs      = ["get", "watch", "list"]
    api_groups = [""]
    resources  = ["services", "endpoints", "pods", "nodes", "namespaces"]
  }

  rule {
    verbs      = ["get", "watch", "list"]
    api_groups = ["extensions", "networking.k8s.io"]
    resources  = ["ingresses"]
  }

  rule {
    verbs      = ["get", "watch", "list"]
    api_groups = ["gateway.networking.k8s.io"]
    resources  = ["gateways", "httproutes", "tlsroutes", "tcproutes", "udproutes"]
  }

  depends_on = [var.node_pool]
}

resource "kubernetes_cluster_role_binding" "external_dns_viewer" {
  metadata {
    name = "external-dns-viewer"

    labels = {
      "app.kubernetes.io/name" = "external-dns"
    }
  }

  subject {
    kind      = "ServiceAccount"
    name      = kubernetes_service_account.external_dns.metadata[0].name
    namespace = kubernetes_service_account.external_dns.metadata[0].namespace
  }

  role_ref {
    api_group = "rbac.authorization.k8s.io"
    kind      = "ClusterRole"
    name      = "external-dns"
  }

  depends_on = [var.node_pool]
}

resource "kubernetes_deployment" "external_dns" {
  metadata {
    name      = "external-dns"
    namespace = local.external_dns_namespace

    labels = {
      "app.kubernetes.io/name" = "external-dns"
    }
  }

  spec {
    selector {
      match_labels = {
        "app.kubernetes.io/name" = "external-dns"
      }
    }

    revision_history_limit = 0

    template {
      metadata {
        labels = {
          "app.kubernetes.io/name" = "external-dns"
        }
      }

      spec {
        container {
          name  = "external-dns"
          image = "registry.k8s.io/external-dns/external-dns:v0.15.1"
          args = [
            "--source=service",
            "--source=ingress",
            "--source=gateway-httproute",
            # "--source=gateway-tlsroute",
            # "--source=gateway-tcproute",
            # "--source=gateway-udproute",
            "--domain-filter=${local.external_dns_domain_filter}",
            "--provider=google",
            "--log-format=json",
            "--registry=txt",
            "--txt-owner-id=k8s-${var.cluster.name}",
            # "--log-level=debug",
          ]
        }

        service_account_name = kubernetes_service_account.external_dns.metadata[0].name
      }
    }

    strategy {
      type = "Recreate"
    }
  }

  depends_on = [var.node_pool, time_sleep.wait_service_cleanup]
}
